//
//	SPI methods.
//
//	Copyright 2014 Mark Stevens
//
#include "spi.h"

//
//	Storage for the SPI data.
//
uint8_t buffer[SPI_BUFFER_LENGTH];
int bufferIndex = 0;

//
//	Initialise SPI
//
void InitialiseSPI()
{
	SPI_InitTypeDef SPI_InitStruct;
	NVIC_InitTypeDef NVIC_InitStructure;
	DMA_InitTypeDef DMA_InitStructure;
	//
	// 	Connect SPI1 pins to SPI alternate function.
	//
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource5, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_SPI1);
	//
	//	Set PE6 high as we will be using active low for the
	//	device select.
	//
	GPIOE->BSRRL |= GPIO_Pin_6;
	//
	// 	Enable the SPI peripheral clock.
	//
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
	//
	//	Configure SPI1 in Mode 0:
	//		CPOL = 0 --> clock is low when idle
	//		CPHA = 0 --> data is sampled at the first edge
	//
	//	SPI Master mode, 8 bits of data, clock prescalar is 128, MSB is
	//	transmitted first.
	//
	SPI_InitStruct.SPI_Direction = SPI_Direction_1Line_Tx;// SPI_Direction_2Lines_FullDuplex;
	SPI_InitStruct.SPI_Mode = SPI_Mode_Master;
	SPI_InitStruct.SPI_DataSize = SPI_DataSize_8b;
	SPI_InitStruct.SPI_CPOL = SPI_CPOL_Low;
	SPI_InitStruct.SPI_CPHA = SPI_CPHA_1Edge;
	SPI_InitStruct.SPI_NSS = SPI_NSS_Soft | SPI_NSSInternalSoft_Set;
	SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_256;
	SPI_InitStruct.SPI_FirstBit = SPI_FirstBit_MSB;
	SPI_Init(SPI1, &SPI_InitStruct);
	//
	//	Configure the DMA controller
	//
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);
	DMA_StructInit(&DMA_InitStructure);
	DMA_InitStructure.DMA_Channel = DMA_Channel_3;
	DMA_InitStructure.DMA_PeripheralBaseAddr  = (uint32_t) &(SPI1->DR);
	DMA_InitStructure.DMA_Memory0BaseAddr  = (uint32_t) &buffer;
	DMA_InitStructure.DMA_DIR = DMA_DIR_MemoryToPeripheral;
	DMA_InitStructure.DMA_BufferSize  = SPI_BUFFER_LENGTH;
	DMA_InitStructure.DMA_PeripheralInc  = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc  = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize  = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_MemoryDataSize  = DMA_MemoryDataSize_Byte;
	DMA_InitStructure.DMA_Mode  = DMA_Mode_Normal;
	DMA_InitStructure.DMA_Priority  = DMA_Priority_High;
	DMA_InitStructure.DMA_FIFOMode  = DMA_FIFOMode_Disable;
	DMA_Init(DMA2_Stream5, &DMA_InitStructure);
	DMA_ITConfig(DMA2_Stream5, DMA_IT_TC, ENABLE);

	NVIC_InitStructure.NVIC_IRQChannel = DMA2_Stream5_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	//
	//	Enable SPI.
	//
	SPI_Cmd(SPI1, ENABLE);
}

//
//	Transmit and receive a single byte of data
//	polling the registers.
//
uint8_t SPISend(uint8_t data)
{
	//
	//	Setting the Data Register (DR) transmits the byte of data on MOSI.
	//
	SPI1->DR = data;
	//
	//	Wait until the data has been transmitted.
	//
	while (!(SPI1->SR & SPI_I2S_FLAG_TXE));
	//
	//	Wait for any data on MISO pin to be received.
	//
	while (!(SPI1->SR & SPI_I2S_FLAG_RXNE));
	//
	//	All data transmitted/received but SPI may be busy so wait until done.
	//
	while (SPI1->SR & SPI_I2S_FLAG_BSY);
	//
	//	Return the data received on MISO pin.
	//
	return(SPI1->DR);
}

//
//	Process the interrupts for SPI1.
//
void SPI1_IRQHandler()
{
	//
	//	If TX buffer is empty then transmit the next byte.
	//
	if (SPI1->SR & SPI_I2S_FLAG_TXE)
	{
		if (bufferIndex < SPI_BUFFER_LENGTH)
		{
			SPI1->DR = buffer[bufferIndex++];
		}
	}
	//
	//	If SPI is not busy then we have finished sending data
	//	so turn off this interrupt.
	//
	if (!(SPI1->SR & SPI_I2S_FLAG_BSY))
	{
		SPI_I2S_ITConfig(SPI1, SPI_I2S_IT_TXE, DISABLE);
		GPIOE->BSRRL |= GPIO_Pin_6;
	}
	//
	//	Clear the interrupt pending bit otherwise this interrupt
	//	will be regenerated.
	//
	SPI_I2S_ClearITPendingBit(SPI1, SPI_I2S_IT_TXE);
}

//
//	SPI DMA handler.
//
void DMA2_Stream5_IRQHandler()
{
	//
	// 	Test if DMA Stream Transfer Complete interrupt
	//
	if (DMA_GetITStatus(DMA2_Stream5, DMA_IT_TCIF5) == SET)
	{
		DMA_ClearITPendingBit(DMA2_Stream5, DMA_IT_TCIF5);
		//
		//	The following is required to ensure that the chip select is not
		//	changed while data is still being transmitted.
		//
		while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_BSY) == SET);
		//
		//	Now set chip select to high.
		//
		GPIOE->BSRRL |= GPIO_Pin_6;
	}
}
